/*************************************************************************************

  phy_link_test
  
  Test program to check propoer function of 'Ethernet PHY Link detection' Master-Brick
  firmware modification

*************************************************************************************/


// ##### ADD THESE LINKER PARAMETERS TO YOUR PROJECT SETTINGS! #####

// -lws2_32 -ladvapi32 -pthread

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include "ip_connection.h"
#include "brick_master.h"


#define HOST "localhost"
#define PORT 4223
#define MASTER_UID "6kpXuV" 	// Change to the UID of your Master Brick


// PHY link monitoring interval in milliseconds
#define LINK_MON_INTERVAL 200


// Declarations
void *check_ethernet_link(void *arg);
Master test_master;


// Functions called according to Ethernet link status
void notify_eth_link_up(void) {
	
	printf("\nEthernet connected, link is up!\n");
	
}

void notify_eth_link_down(void) {
	
	printf("\nEthernet disconnected!\n");
	
}




int main(int argc, char *argv[]) {
	
    // Create IP connection
    IPConnection ipcon;
    ipcon_create(&ipcon);
    // Create device objects
    master_create(&test_master, MASTER_UID, &ipcon);
    // Connect to brickd
    if(ipcon_connect(&ipcon, HOST, PORT) < 0) {
        fprintf(stderr, "Could not connect\n");
        return 1;
    }
    // Don't use device before ipcon is connected
    bool eth_ext_available = false;
    master_is_ethernet_present(&test_master, &eth_ext_available);
    if (true == eth_ext_available) {
    	printf("Ethernet extension available...\n\n");
		printf("\n----- ETHERNET LINK STATUS -----\n\n");
	    
	    // Create thread for monitoring of Ethernet connection
		pthread_t eth_con_thread;
		pthread_create(&eth_con_thread, NULL, check_ethernet_link, NULL);
		// Do nothing else...
		while(1);		    	
	}
	else {
		printf("No Ethernet extension found!\nPress any key to exit...");
		scanf("");
	}
    master_destroy(&test_master);
    ipcon_destroy(&ipcon); // Calls ipcon_disconnect internally
	return 0;
}



// Thread function: Checking for a broken Ethernet link
void *check_ethernet_link(void *arg) {

	// State variables used to detect a change in the ethernet connection status
	bool prev_link = true;
	bool curr_link = true;
		
	while(1) {
		
		master_is_ethernet_link_up(&test_master, &curr_link);
		// In case a change is detected, execute corresponding callback function	
		if ((true == prev_link) && (false == curr_link)) {
			prev_link = curr_link;
			// 'No Ethernet link'	
			notify_eth_link_down();				
		}
		else if ((false == prev_link) && (true == curr_link)) {
			prev_link = curr_link;
			// 'Ethernet link is up'
			notify_eth_link_up();				
		}
		else {
			// do nothing in this case...
		}
		usleep(LINK_MON_INTERVAL*1000);
		//printf("Checking...\n");	
	}	
	return 0;	
}
